@@ -101,12 +101,20 @@ IOService * IOFramebufferUserClient::getService( void )
 IOReturn IOFramebufferUserClient::clientMemoryForType( UInt32 type,
         IOOptionBits * flags, IOMemoryDescriptor ** memory )
 {
-    IOMemoryDescriptor *	mem;
-    IOReturn		err;
+    static bool		 havePublishedResource;
+    IOMemoryDescriptor * mem;
+    IOReturn		 err;
 
     switch (type)
     {
         case kIOFBCursorMemory:
+
+	    if (!havePublishedResource)
+	    {
+		havePublishedResource = true;
+		publishResource("WindowServer");
+	    }
+
             mem = owner->sharedCursor;
             mem->retain();
             break;
@@ -247,31 +255,24 @@ IOFramebufferSharedUserClient * IOFramebufferSharedUserClient::withTask(
 
 bool IOFramebufferSharedUserClient::start( IOService * _owner )
 {
-    static const IOExternalMethod methodTemplate[] = {
-            };
-
     if (!super::start(_owner))
         return (false);
 
     owner = (IOFramebuffer *) _owner;
-
-    bcopy( methodTemplate, externals, sizeof( methodTemplate ));
+    owner->sharedConnect = this;
 
     return (true);
 }
 
 void IOFramebufferSharedUserClient::free( void )
 {
-    retain();
-    retain();
     owner->sharedConnect = 0;
-    detach( owner);
     super::free();
 }
 
 void IOFramebufferSharedUserClient::release() const
 {
-    super::release(2);
+    super::release();
 }
 
 IOReturn IOFramebufferSharedUserClient::clientClose( void )
@@ -318,11 +319,16 @@ IOReturn IOFramebufferSharedUserClient::getNotificationSemaphore(
     return (owner->getNotificationSemaphore(interruptType, semaphore));
 }
 
-IOExternalMethod * IOFramebufferSharedUserClient::getExternalMethodForIndex( UInt32 index )
+IOExternalMethod * IOFramebufferSharedUserClient::getTargetAndMethodForIndex(
+    IOService ** targetP, UInt32 index )
 {
-    if (index < (sizeof(externals) / sizeof(externals[0])))
-        return (externals + index);
-    else
+    static const IOExternalMethod methodTemplate[] = {
+    };
+
+    if (index > (sizeof(methodTemplate) / sizeof(methodTemplate[0])))
         return (NULL);
+
+    *targetP = this;
+    return ((IOExternalMethod *)(methodTemplate + index));
 }
 
