@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -158,6 +155,19 @@ extern OSStatus    CallTVector(
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+class IOPEFContainer : public OSData
+{
+    OSDeclareDefaultStructors(IOPEFContainer)
+
+    OSData * fContainer;
+    OSData * fDescription;
+
+public:
+    static IOPEFContainer * withData(OSData * data, OSData * description);
+    virtual void free( void );
+    virtual bool serialize(OSSerialize *s) const;
+};
+
 class IONDRV : public OSObject
 {
     OSDeclareAbstractStructors(IONDRV)
@@ -181,18 +191,18 @@ private:
     struct IOTVector *		fDoDriverIO;
     struct DriverDescription *	fDriverDesc;
     char			fName[64];
-    kmod_info_t *		fKModInfo;
     kmod_t			fKModID;
 
 public:
     static void initialize( void );
-    static IONDRV * instantiate( IORegistryEntry * regEntry,
+    static IOPEFNDRV * instantiate( IORegistryEntry * regEntry,
                                  IOLogicalAddress container,
                                  IOByteCount containerSize,
+				 bool checkDate,
                                  IONDRVUndefinedSymbolHandler handler,
                                  void * self );
 
-    static IONDRV * fromRegistryEntry( IORegistryEntry * regEntry, OSData * newPEF,
+    static IOPEFNDRV * fromRegistryEntry( IORegistryEntry * regEntry, OSData * newPEF,
                                         IONDRVUndefinedSymbolHandler handler, void * self);
 
     virtual void free( void );
